# -*- coding: utf-8 -*-
__title__ = "Django Rest Framework JWT Devices"
__version__ = "1.2.2"
__author__ = "Michal Proszek"
__license__ = "MIT"
__copyright__ = "Copyright 2017-2018 Arabella"

# Version synonym
VERSION = __version__
